#include <math.h>

#include <qcheckbox.h>
#include <qcombobox.h>
#include <qdialog.h>
#include <qpopupmenu.h>
#include <qmenubar.h>
#include <qkeycode.h>
#include <qfile.h>
#include <qfiledialog.h>
#include <qlineedit.h>
#include <qlistbox.h>
#include <qpixmap.h>
#include <qstatusbar.h>
#include <qmessagebox.h>
#include <qapplication.h>
#include <qprogressdialog.h>
#include <qtabwidget.h>
#include <qtable.h>
#include <qwidget.h>
#include <qwidgetfactory.h>
#include <qpainter.h>
#include <qprinter.h>
#include <qspinbox.h>
#include <qtextedit.h>

#include "xpm/fileopen.xpm"
#include "xpm/filesave.xpm"
#include "xpm/fileprint.xpm"

#include "appwin.h"



//**************************************************************************************************
//  class AppWin
//**************************************************************************************************


AppWin::AppWin()
    : QMainWindow( 0, "ddplot", WDestructiveClose )
{
  QPixmap openIcon, printIcon, saveIcon;
  int pos;

  tabWidget = NULL;

  loadSettings();

  openIcon = QPixmap( fileopen );
  printIcon = QPixmap( fileprint );
  saveIcon = QPixmap( filesave );

  resize( WINWIDTH, WINHEIGHT );
  

  //----------------------------------------------------------------------------

  mfile = new QPopupMenu( this );
  Q_CHECK_PTR( mfile );
  menuBar()->insertItem( "&File", mfile );

  OpenID     = mfile->insertItem( openIcon, "&Open...", this, SLOT(open()), CTRL+Key_O );
  SaveID     = mfile->insertItem( saveIcon, "&Save as .dd...", this, SLOT(saveDD()), CTRL+Key_S );
  CloseID    = mfile->insertItem( "&Close", this, SLOT(close()), CTRL+Key_F3 );
  CloseAllID = mfile->insertItem( "C&lose all", this, SLOT(closeAll()) );
  mfile->insertSeparator();
  PrintID    = mfile->insertItem( printIcon, "&Print...", this, SLOT(print()), CTRL+Key_P ); 
  mfile->insertSeparator();

  mexport = new QPopupMenu( mfile );
  Q_CHECK_PTR( mexport );
  ExportID = mfile->insertItem( "&Export", mexport );

    ExpXYZID = mexport->insertItem( "to XYZ format (JMol)", this, SLOT(exportXYZ()) );
    ExpCFGID = mexport->insertItem( "to CFG format (AtomEye)", this, SLOT(exportCFG()) );

  AnimateID  = mfile->insertItem( "&Animate all frames to .GIF...", this, SLOT(animate()) );
  mfile->insertSeparator();
  QuitID     = mfile->insertItem( "&Quit", this, SLOT(quit()), CTRL+Key_Q );

  //----------------------------------------------------------------------------

  mview = new QPopupMenu( this );
  Q_CHECK_PTR( mview );
  menuBar()->insertItem( "&View", mview );

  ZoomInID    = mview->insertItem( "Zoom &in", this, SLOT(zoomIn()), Key_Plus );  
  ZoomOutID   = mview->insertItem( "Zoom &out", this, SLOT(zoomOut()), Key_Minus );  
  FitWinID    = mview->insertItem( "&Fit window", this, SLOT(fitWin()), Key_Asterisk );

  mview->insertSeparator();

  PanRightID  = mview->insertItem( "Pan &right", this, SLOT(panRight()), SHIFT+Key_Right );  
  PanLeftID   = mview->insertItem( "Pan &left", this, SLOT(panLeft()), SHIFT+Key_Left );   
  PanUpID     = mview->insertItem( "Pan &up", this, SLOT(panUp()), SHIFT+Key_Up );  
  PanDownID   = mview->insertItem( "Pan &down", this, SLOT(panDown()), SHIFT+Key_Down );  

  mview->insertSeparator();

  PanFRightID = mview->insertItem( "Pan fast &right", this, SLOT(panFastRight()), CTRL+Key_Right );  
  PanFLeftID  = mview->insertItem( "Pan fast &left", this, SLOT(panFastLeft()), CTRL+Key_Left );   
  PanFUpID    = mview->insertItem( "Pan fast &up", this, SLOT(panFastUp()), CTRL+Key_Up );  
  PanFDownID  = mview->insertItem( "Pan fast &down", this, SLOT(panFastDown()), CTRL+Key_Down );

  //----------------------------------------------------------------------------

  mplot = new QPopupMenu( this );
  Q_CHECK_PTR( mplot );
  menuBar()->insertItem( "&Plot", mplot );

  SelPlotsID  = mplot->insertItem( "&Select plots...", this, SLOT(selectPlots()), CTRL+Key_S );

  mplot->insertSeparator();

  UnrelViewID = mplot->insertItem( "&Unrelaxed configuration", this, SLOT(plotUnrelaxed()), Key_U );  
  RelViewID   = mplot->insertItem( "&Relaxed configuration", this, SLOT(plotRelaxed()), Key_R );
  CompViewID  = mplot->insertItem( "&Composite view (relaxed+unrelaxed)", this, SLOT(plotComposite()), Key_C );
  mplot->insertSeparator();

  MagDispID   = mplot->insertItem( "P&rint magnitudes of displacements", this, SLOT(printMagDisp()), Key_M );
  NumAtomID   = mplot->insertItem( "Pr&int numbers of atoms", this, SLOT(printAtomNum()), Key_N );

  mplot->insertSeparator();

  EdgeCompID  = mplot->insertItem( "&Edge component", this, SLOT(plotEdgeComponent()), Key_E );  
  ScrewCompID = mplot->insertItem( "&Screw component", this, SLOT(plotScrewComponent()), Key_S );

  mplot->insertSeparator();

  DAtLayersID = mplot->insertItem( "&Distinguish atomic layers", this, SLOT(plotAtomLayers()), Key_L );  
  DAtTypesID  = mplot->insertItem( "D&istinguish atom types", this, SLOT(plotAtomTypes()), Key_T );  
  InertAtomID = mplot->insertItem( "Sho&w inert atoms", this, SLOT(plotInertAtoms()) );  

  mplot->insertSeparator();

  DCenterID   = mplot->insertItem( "S&how dislocation center", this, SLOT(disloCenter()) );
  SPlanesID   = mplot->insertItem( "S&how traces of planes...", this, SLOT(showPlanes()) );

  mplot->insertSeparator();

  ShowGbID    = mplot->insertItem( "Show &grain boundary", this, SLOT(showGb()) );

  mplot->insertSeparator();

  RelaxInfoID = mplot->insertItem( "&Display relaxation info...", this, SLOT(relaxInfo()), Key_I );

  mplot->setItemChecked( UnrelViewID, ATOMPOS==UNRELAXED );
  mplot->setItemChecked( RelViewID, ATOMPOS==RELAXED );
  mplot->setItemChecked( CompViewID, ATOMPOS==COMPOSITE );
  mplot->setItemChecked( ShowGbID, SHOWGB );

  mplot->setItemChecked( MagDispID, PRINT_MAG_DISP );

  mplot->setItemChecked( EdgeCompID, DISP_COMPONENT==EDGE );
  mplot->setItemChecked( ScrewCompID, DISP_COMPONENT==SCREW );

  mplot->setItemChecked( DAtLayersID, PLOT_TYPE==PLOT_ATOM_LAYERS );
  mplot->setItemChecked( DAtTypesID, PLOT_TYPE==PLOT_ATOM_TYPES );

  mplot->setItemChecked( DCenterID, SHOW_DCENTER );

  //----------------------------------------------------------------------------

  mlayers = new QPopupMenu( this );
  Q_CHECK_PTR( mlayers );
  menuBar()->insertItem( "&Layers", mlayers );

  SetZLayID   = mlayers->insertItem( "Show/hide &particular layer(s)...", this, SLOT(selectZLayers()) );

  mlayers->insertSeparator();

  ShowZLayID  = mlayers->insertItem( "Show a list of &Z-layers", this, SLOT(showZLayers()), Key_Z );

  mlayers->setItemChecked( ShowZLayID, SHOW_ZLAYERS );

  //----------------------------------------------------------------------------

  mtran = new QPopupMenu( this );
  Q_CHECK_PTR( mtran );
  menuBar()->insertItem( "&Transformations", mtran );

  CoordSysID  = mtran->insertItem( "&Give the coordinate system of the block...", this, SLOT(coordSys()) );

  mtran->insertSeparator();

  XFlipID  = mtran->insertItem( "Flip about the X-axis", this, SLOT(flipBlock(int)) );
  YFlipID  = mtran->insertItem( "Flip about the Y-axis", this, SLOT(flipBlock(int)) );  
  ZFlipID  = mtran->insertItem( "Flip about the Z-axis", this, SLOT(flipBlock(int)) );

  mtran->insertSeparator();

  TransformID = mtran->insertItem( "User-defined transformation...", this, SLOT(transform()) );

  //----------------------------------------------------------------------------

  mappear = new QPopupMenu( this );
  Q_CHECK_PTR( mappear );
  menuBar()->insertItem( "&Appearance", mappear );

  CrystPlotID = mappear->insertItem( "&Crystal structure plotting...", this, SLOT(crystalPlot()) );
  ScalePosID  = mappear->insertItem( "Scale &atomic positions...", this, SLOT(scalePos()) );  
  
  mappear->insertSeparator();

  ArrPlotID   = mappear->insertItem( "&Arrows plotting...", this, SLOT(arrowsPlot()) );
  ScaleArrID  = mappear->insertItem( "&Scale lengths of arrows...", this, SLOT(scaleArrow()) );

  mappear->insertSeparator();

  ProjVectID  = mappear->insertItem( "Set &projection vector...", this, SLOT(projVector()), Key_P );  
  ArrNeighID  = mappear->insertItem( "P&lot arrows between which neighbors...", this, SLOT(arrNeighbors()) ); 

  //----------------------------------------------------------------------------

  mhelp = new QPopupMenu( this );
  Q_CHECK_PTR( mhelp );
  menuBar()->insertItem( "&Help", mhelp );

  AboutID = mhelp->insertItem( "&About...", this, SLOT(about()), CTRL+Key_A );

  //----------------------------------------------------------------------------

  enableMenuItems(false);

  printer = new QPrinter( QPrinter::PrinterResolution );
  printer->setColorMode(QPrinter::Color);
  printer->setPageSize(QPrinter::Letter);

  // Loading of the plot file entered through the command-line. If the extension of the
  // file is not .dd, the program automatically supposses that it is in the old .plt
  // format.
  if (PLOT_FILE!="") {
    loadFile( PLOT_FILE );
  }
}


AppWin::~AppWin()
{
  delete( printer );
}


void AppWin::closeEvent( QCloseEvent * )
{
  quit();
}


void AppWin::resizeEvent( QResizeEvent * )
{
  if (tabWidget==NULL)
    return;

  tabWidget->setGeometry( 0, 23, width(), height()-40 ); 
}


